/***
 * TMapUnit = マップユニット管理用レコード
 *  x : integer X座標 小数可能
 *  y : integer Y座標 小数可能
 *  graphic : string グラフィックファイル名
 *  obj : object 画像を記録したimgタグのオブジェクト nullなら生成されていない
 */
var MapUnits = [];
var GridFixed = true;

function reloadMapChip() {
	var l = MapUnits.length,aunit;
	if (null === CELL_SIZE) {
		return;
	}
	for (var i=0;i<l;i++) {
		if (null === MapUnits[i]['obj'] || void(0) === MapUnits[i]['obj']) {
			// 生成する
			MapUnits[i]['obj'] = $('<img src="' + MapUnits[i]['graphic'] + '" style="' +
				'left:' + (CELL_SIZE * MapUnits[i]['x']) + 'px;top:' + (CELL_SIZE * MapUnits[i]['y']) + 'px;">');
			MapUnits[i]['div'] = $('<div class="img" style="position: absolute;z-index:1000"></div>');
			$(MapUnits[i]['div']).prepend(MapUnits[i]['obj']);
			MapUnits[i]['div'].prop('title',MapUnits[i]['title']);
			
			if (MapUnits[i]['id'] != '' && MapUnits[i]['id'] !== void(0)) {
				MapUnits[i]['span'] = $('<span class="img_id_span">' + sanitaize.encode(MapUnits[i]['id']) + '</span>');
				$(MapUnits[i]['div']).append(MapUnits[i]['span']);
			}
			
			if (MapUnits[i]['title'] != '' && MapUnits[i]['title'] !== void(0)) {
				MapUnits[i]['title_div'] = $('<div class="img_title_div"><span class="img_title_span">' +
					sanitaize.encode(MapUnits[i]['title']) + '</span></div>');
				$(MapUnits[i]['div']).append(MapUnits[i]['title_div']);
			}

			MapUnits[i]['onload'] = new Function('unitImageLoadPlace(' + i + ');');
			MapUnits[i]['obj'][0].onload = MapUnits[i]['onload'];
			
			$('#map_container').append(MapUnits[i]['div']);
		}
		if (MapUnits[i]['obj'][0].complete) {
			unitImageLoadPlace(i);
		}
		
		MapUnits[i]['div'].draggable({
			appendTo : MapUnits[i]['obj'][0],
			stop : unitMove,
			cancel : '.img_title_div'
		});
	}
	
	$().contextMenu({
		selector : '#map_container img',
		callback: MapUnitMenuCallback,
		items: {
			"un_change": {name: "設定を変更する", icon: ""},
			"sep": '----',
			"un_up": {name: "元通りにする", icon: ""},
			"un_right": {name: "右に倒す", icon: ""},
			"un_left": {name: "左に倒す", icon: ""},
			"un_down": {name: "反転する", icon: ""},
		}
	});
}


function unitImageLoadPlace(i) {
	var target = MapUnits[i]['obj'][0];
	var w = target.width, h = target.height,l;
	if (void(0) === MapUnits[i]['width']) {
		MapUnits[i]['width'] = w;
		MapUnits[i]['height'] = h;
	} else {
		w = MapUnits[i]['width'];
		h = MapUnits[i]['height'];
	}
	if (w < h) {
		l = CELL_SIZE * w / h;
		MapUnits[i]['xadd'] = (CELL_SIZE - l) / 2;
		MapUnits[i]['yadd'] = 0;
		w = l;
		h = CELL_SIZE;
	} else if (w > h) {
		l = CELL_SIZE * h / w;
		MapUnits[i]['xadd'] = 0;
		MapUnits[i]['yadd'] = (CELL_SIZE - l) / 2;
		w = CELL_SIZE;
		h = l;
	} else {
		MapUnits[i]['xadd'] = 0;
		MapUnits[i]['yadd'] = 0;
		w = CELL_SIZE;
		h = CELL_SIZE;
	}
	
	$(target).css({
		left:(CELL_SIZE * MapUnits[i]['x'] + MapUnits[i]['xadd']) + 'px',
		top: (CELL_SIZE * MapUnits[i]['y'] + MapUnits[i]['yadd']) + 'px',
		width : w,
		height: h
	});
	var target = MapUnits[i]['div'][0];
	$(target).css({
		left:(CELL_SIZE * MapUnits[i]['x'] + MapUnits[i]['xadd']) + 'px',
		top: (CELL_SIZE * MapUnits[i]['y'] + MapUnits[i]['yadd']) + 'px',
		width : w,
		height: h
	});
}

function unitMove(e, ui) {
	var dest = e.target,l = MapUnits.length,aunit,x,y;
	var hex_mode = $('#map').hasClass('hex');
	for (var i=0;i<l;i++) {
		if (dest == MapUnits[i]['div'][0]) {
			x = (ui['position']['left'] - MapUnits[i]['xadd']) / CELL_SIZE;
			y = (ui['position']['top'] - MapUnits[i]['yadd']) / CELL_SIZE;
			if (GridFixed) {
				x = Math.round(x);
				if (x & 1 === 1 && hex_mode) {
					y = Math.round(y - 0.5);
					$(dest).css({
						left:(CELL_SIZE * x + MapUnits[i]['xadd']) + 'px',
						top: (CELL_SIZE * y + MapUnits[i]['yadd'] + (CELL_SIZE >> 1)) + 'px'
					});
					y += 0.5;
				} else {
					y = Math.round(y);
					$(dest).css({
						left:(CELL_SIZE * x + MapUnits[i]['xadd']) + 'px',
						top: (CELL_SIZE * (y) + MapUnits[i]['yadd']) + 'px'
					});
				}
			}
			MapUnits[i]['x'] = x;
			MapUnits[i]['y'] = y;
		}
	}
}

function MapUnitFind(img) {
	var l = MapUnits.length;
	for (var i=0;i<l;i++) {
		if (img == MapUnits[i]['obj'][0]) {
			return i;
		}
	}
}

function MapUnitMenuCallback(key, options) {
	var id = MapUnitFind(this[0]);
	if (key === "un_change") {
		// 変更
	} else if (key === "un_up") {
		this.css({transform: ''});
	} else if (key === "un_right") {
		this.css({transform: 'rotate(90deg)'});
	} else if (key === "un_left") {
		this.css({transform: 'rotate(270deg)'});
	} else if (key === "un_down") {
		this.css({transform: 'rotate(180deg)'});
	}
	
	var target = $(this[0]);
	$(this[0]).css({
		left:(Math.round(parseFloat(this[0].style.left))) + 'px',
		top: (Math.round(parseFloat(this[0].style.top))) + 'px'
	});
}

// Map Add Dialog

// Unit Placer Dialog

var sanitaize = {
	encode : function (str) {
		return str.replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;').replace(/"/g, '&quot;').replace(/'/g, '&#39;');
	},

	decode : function (str) {
		return str.replace(/&lt;/g, '<').replace(/&gt;/g, '>').replace(/&quot;/g, '"').replace(/&#39;/g, '\'').replace(/&amp;/g, '&');
	}
};

function updateImg(is_img_or_filename) {
	if (true !== is_img_or_filename) {
		$('#unitplacer_image').val(is_img_or_filename);
	} else {
		is_img_or_filename = $('#unitplacer_image').val();
	}
	var img = new Image,src = sanitaize.encode(is_img_or_filename);
	
	img.onload = function(){
		if (img === null) {
			return;
		}
		var width = img.width;
		var height = img.height;
		$('#unitplacer_img')[0].src = src;
		if (width > height) {
			$('#unitplacer_img')[0].width = 150;
			$('#unitplacer_img')[0].height = 150 * height / width;
		} else {
			$('#unitplacer_img')[0].height = 150;
			$('#unitplacer_img')[0].width = 150 * width / height;
		}
		img = null;
	};
	img.onerror = function() {
		img = null;
		$('#unitplacer_img')[0].src = '';
	};
	img.src = src;
	if (img !== null && img.complete) {
		var width = img.width;
		var height = img.height;
		$('#unitplacer_img')[0].src = src;
		if (width > height) {
			$('#unitplacer_img')[0].width = 150;
			$('#unitplacer_img')[0].height = 150 * height / width;
		} else {
			$('#unitplacer_img')[0].height = 150;
			$('#unitplacer_img')[0].width = 150 * width / height;
		}
		img = null;
	}
	
}
window.addEventListener("message", receiveMessage, false);
var Calling_origin = 'null';
var MAKEN_status = [{}];

function receiveMessage(event) {
	if (event.origin !== Calling_origin) {
		return;
	}
	alert(1);
	if (event.data !== true && typeof event.data === 'string') {
		updateImg(event.data);
	}
}
var dialoggrid;

$().ready(function() {
	$('#map_adder').draggable();
	$('#map_adder_close').click(function(){ $('#map_adder').hide(); });
	$('#map_adder img').draggable({
		'stop' : function(e,ui) {
			var BaseCell = $('#map_container')[0].getBoundingClientRect();
			var NowPoint = this.getBoundingClientRect();
			MapUnits.push({
				'graphic':'pawnWhite.png',
				'x' : Math.floor((NowPoint['x'] - BaseCell['x']) / CELL_SIZE),
				'y' : Math.floor((NowPoint['y'] - BaseCell['y']) / CELL_SIZE),
				'id': $('#unit_adder_id').val(),
			});
			this.style.left = 0;
			this.style.top = 0;
			
			if (! $('#unit_adder_multi')[0].checked) {
				$('#map_adder').hide();
			} else {
				$('#unit_adder_id').val(($('#unit_adder_id').val() - 0) + 1);
			}
			reloadMapChip();
		}
	});
});

MAKEN_status = [{'hp':0,'/hp':0,'mp':0,'/mp':0,'毒':false}];


$().ready(function() {
	MapUnits.push({x:0,y:0,graphic:'pawnWhite.png',title:'<b>"コマ1"%&</b>'});
	MapUnits.push({x:1,y:0,graphic:'pawnWhite.png',title:'コマ2',id:'<script>alert(1)</script>'});
	reloadMapChip();
});

$().ready(function() {
	var columns = [
		{id: "hp", name: "HP", field: "hp", width: 70, formatter: ParamFormatter, editor: ParamEditor},
		{id: "mp", name: "MP", field: "mp", width: 70, formatter: ParamFormatter, editor: ParamEditor},
		{id: "毒", name: "毒", field: "毒", width: 30, formatter: Slick.Formatters.Checkmark, editor: Slick.Editors.Autocheck}
	];
	var options = {
		editable: true,
		enableAddRow: false,
		autoHeight:true
	};

	dialoggrid = new Slick.Grid("#unitplacer_append_status", MAKEN_status, columns, options);
	$('#unitplacer_append_status .slick-viewport').height('1.5em');
	spinner = $( "#unitplacer_size" ).spinner();
	$('#unitplacer_append_status .slick-viewport').height('1.5em');
	$("#unitplacer_container").dialog({
		autoOpen: false,
		modal:true,
		width:600,
		buttons: {
			'追加': function() {
				if (jQuery('#unitplacer_name').val() == '') {
					return false;
				}
				jQuery( this ) . dialog( 'close' );
			},
			'キャンセル': function() {
				jQuery( this ) . dialog( 'close' );
			}
		}
	});
});
